@@ -1,6 +1,6 @@
 ﻿/*
  * Carla Plugin Host
- * Copyright (C) 2011-2019 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -350,6 +350,9 @@ typedef struct {
     int stride;
 } CarlaInlineDisplayImageSurface;
 
+/*! Opaque data type for CarlaHost API calls */
+typedef struct _CarlaHostHandle* CarlaHostHandle;
+
 /* ------------------------------------------------------------------------------------------------------------
  * Carla Host API (C functions) */
 
@@ -385,12 +388,17 @@ CARLA_EXPORT const EngineDriverDeviceInfo* carla_get_engine_driver_device_info(u
  */
 CARLA_EXPORT bool carla_show_engine_driver_device_control_panel(uint index, const char* name);
 
+/*!
+ * Create a global host handle for standalone application usage.
+ */
+CARLA_EXPORT CarlaHostHandle carla_standalone_host_init(void);
+
 #ifdef __cplusplus
 /*!
  * Get the currently used engine, may be NULL.
  * @note C++ only
  */
-CARLA_EXPORT CarlaEngine* carla_get_engine(void);
+CARLA_EXPORT CarlaEngine* carla_get_engine_from_handle(CarlaHostHandle handle);
 #endif
 
 /*!
@@ -399,14 +407,18 @@ CARLA_EXPORT CarlaEngine* carla_get_engine(void);
  * @param driverName Driver to use
  * @param clientName Engine master client name
  */
-CARLA_EXPORT bool carla_engine_init(const char* driverName, const char* clientName);
+CARLA_EXPORT bool carla_engine_init(CarlaHostHandle handle, const char* driverName, const char* clientName);
 
 #ifdef BUILD_BRIDGE
 /*!
  * Initialize the engine in bridged mode.
  */
-CARLA_EXPORT bool carla_engine_init_bridge(const char audioBaseName[6+1], const char rtClientBaseName[6+1], const char nonRtClientBaseName[6+1],
-                                           const char nonRtServerBaseName[6+1], const char* clientName);
+CARLA_EXPORT bool carla_engine_init_bridge(CarlaHostHandle handle,
+                                           const char audioBaseName[6+1],
+                                           const char rtClientBaseName[6+1],
+                                           const char nonRtClientBaseName[6+1],
+                                           const char nonRtServerBaseName[6+1],
+                                           const char* clientName);
 #endif
 
 /*!
@@ -414,85 +426,83 @@ CARLA_EXPORT bool carla_engine_init_bridge(const char audioBaseName[6+1], const
  * This function always closes the engine even if it returns false.
  * In other words, even when something goes wrong when closing the engine it still be closed nonetheless.
  */
-CARLA_EXPORT bool carla_engine_close(void);
+CARLA_EXPORT bool carla_engine_close(CarlaHostHandle handle);
 
 /*!
  * Idle the engine.
  * Do not call this if the engine is not running.
  */
-CARLA_EXPORT void carla_engine_idle(void);
+CARLA_EXPORT void carla_engine_idle(CarlaHostHandle handle);
 
 /*!
  * Check if the engine is running.
  */
-CARLA_EXPORT bool carla_is_engine_running(void);
+CARLA_EXPORT bool carla_is_engine_running(CarlaHostHandle handle);
 
 /*!
  * Get information about the currently running engine.
  */
-CARLA_EXPORT const CarlaRuntimeEngineInfo* carla_get_runtime_engine_info(void);
+CARLA_EXPORT const CarlaRuntimeEngineInfo* carla_get_runtime_engine_info(CarlaHostHandle handle);
 
 #ifndef BUILD_BRIDGE
 /*!
  * Get information about the currently running engine driver device.
  */
-CARLA_EXPORT const CarlaRuntimeEngineDriverDeviceInfo* carla_get_runtime_engine_driver_device_info(void);
+CARLA_EXPORT const CarlaRuntimeEngineDriverDeviceInfo* carla_get_runtime_engine_driver_device_info(CarlaHostHandle handle);
 
 /*!
  * Dynamically change buffer size and/or sample rate while engine is running.
  * @see ENGINE_DRIVER_DEVICE_VARIABLE_BUFFER_SIZE
  * @see ENGINE_DRIVER_DEVICE_VARIABLE_SAMPLE_RATE
  */
-CARLA_EXPORT bool carla_set_engine_buffer_size_and_sample_rate(uint bufferSize, double sampleRate);
+CARLA_EXPORT bool carla_set_engine_buffer_size_and_sample_rate(CarlaHostHandle handle, uint bufferSize, double sampleRate);
 
 /*!
  * Show the custom control panel for the current engine device.
  * @see ENGINE_DRIVER_DEVICE_HAS_CONTROL_PANEL
  */
-CARLA_EXPORT bool carla_show_engine_device_control_panel(void);
+CARLA_EXPORT bool carla_show_engine_device_control_panel(CarlaHostHandle handle);
 #endif
 
 /*!
  * Clear the xrun count on the engine, so that the next time carla_get_runtime_engine_info() is called, it returns 0.
  */
-CARLA_EXPORT void carla_clear_engine_xruns(void);
+CARLA_EXPORT void carla_clear_engine_xruns(CarlaHostHandle handle);
 
 /*!
  * Tell the engine to stop the current cancelable action.
  * @see ENGINE_CALLBACK_CANCELABLE_ACTION
  */
-CARLA_EXPORT void carla_cancel_engine_action(void);
+CARLA_EXPORT void carla_cancel_engine_action(CarlaHostHandle handle);
 
 /*!
  * Tell the engine it's about to close.
  * This is used to prevent the engine thread(s) from reactivating.
  * Returns true if there's no pending engine events.
  */
-CARLA_EXPORT bool carla_set_engine_about_to_close(void);
+CARLA_EXPORT bool carla_set_engine_about_to_close(CarlaHostHandle handle);
 
 /*!
  * Set the engine callback function.
  * @param func Callback function
  * @param ptr  Callback pointer
  */
-CARLA_EXPORT void carla_set_engine_callback(EngineCallbackFunc func, void* ptr);
+CARLA_EXPORT void carla_set_engine_callback(CarlaHostHandle handle, EngineCallbackFunc func, void* ptr);
 
-#ifndef BUILD_BRIDGE
 /*!
  * Set an engine option.
  * @param option   Option
  * @param value    Value as number
  * @param valueStr Value as string
  */
-CARLA_EXPORT void carla_set_engine_option(EngineOption option, int value, const char* valueStr);
-#endif
+CARLA_EXPORT void carla_set_engine_option(CarlaHostHandle handle, EngineOption option, int value, const char* valueStr);
 
 /*!
  * Set the file callback function.
  * @param func Callback function
  * @param ptr  Callback pointer
  */
-CARLA_EXPORT void carla_set_file_callback(FileCallbackFunc func, void* ptr);
+CARLA_EXPORT void carla_set_file_callback(CarlaHostHandle handle, FileCallbackFunc func, void* ptr);
 
 /*!
  * Load a file of any type.
@@ -500,24 +510,36 @@ CARLA_EXPORT void carla_set_file_callback(FileCallbackFunc func, void* ptr);
  * either by direct handling (SF2 and SFZ) or by using an internal plugin (like Audio and MIDI).
  * @see carla_get_supported_file_extensions()
  */
-CARLA_EXPORT bool carla_load_file(const char* filename);
+CARLA_EXPORT bool carla_load_file(CarlaHostHandle handle, const char* filename);
 
 /*!
  * Load a Carla project file.
  * @note Currently loaded plugins are not removed; call carla_remove_all_plugins() first if needed.
  */
-CARLA_EXPORT bool carla_load_project(const char* filename);
+CARLA_EXPORT bool carla_load_project(CarlaHostHandle handle, const char* filename);
 
 /*!
  * Save current project to a file.
  */
-CARLA_EXPORT bool carla_save_project(const char* filename);
+CARLA_EXPORT bool carla_save_project(CarlaHostHandle handle, const char* filename);
 
 #ifndef BUILD_BRIDGE
+/*!
+  * Get the currently set project folder.
+  * @note Valid for both standalone and plugin versions.
+ */
+CARLA_EXPORT const char* carla_get_current_project_folder(CarlaHostHandle handle);
+
+/*!
+ * Get the currently set project filename.
+ * @note Valid only for both standalone version.
+ */
+CARLA_EXPORT const char* carla_get_current_project_filename(CarlaHostHandle handle);
+
 /*!
  * Clear the currently set project filename.
  */
-CARLA_EXPORT void carla_clear_project_filename(void);
+CARLA_EXPORT void carla_clear_project_filename(CarlaHostHandle handle);
 
 /*!
  * Connect two patchbay ports.
@@ -527,63 +549,72 @@ CARLA_EXPORT void carla_clear_project_filename(void);
  * @param portIdB  Input port
  * @see ENGINE_CALLBACK_PATCHBAY_CONNECTION_ADDED
  */
-CARLA_EXPORT bool carla_patchbay_connect(bool external, uint groupIdA, uint portIdA, uint groupIdB, uint portIdB);
+CARLA_EXPORT bool carla_patchbay_connect(CarlaHostHandle handle, bool external, uint groupIdA, uint portIdA, uint groupIdB, uint portIdB);
 
 /*!
  * Disconnect two patchbay ports.
  * @param connectionId Connection Id
  * @see ENGINE_CALLBACK_PATCHBAY_CONNECTION_REMOVED
  */
-CARLA_EXPORT bool carla_patchbay_disconnect(bool external, uint connectionId);
+CARLA_EXPORT bool carla_patchbay_disconnect(CarlaHostHandle handle, bool external, uint connectionId);
+
+/*!
+ * Set the position of a group.
+ * This is purely cached and saved in the project file, Carla backend does nothing with the value.
+ * When loading a project, callbacks are used to inform of the previously saved positions.
+ * @see ENGINE_CALLBACK_PATCHBAY_CLIENT_POSITION_CHANGED
+ */
+CARLA_EXPORT bool carla_patchbay_set_group_pos(CarlaHostHandle handle, bool external,
+                                               uint groupId, int x1, int y1, int x2, int y2);
 
 /*!
  * Force the engine to resend all patchbay clients, ports and connections again.
  * @param external Wherever to show external/hardware ports instead of internal ones.
  *                 Only valid in patchbay engine mode, other modes will ignore this.
  */
-CARLA_EXPORT bool carla_patchbay_refresh(bool external);
+CARLA_EXPORT bool carla_patchbay_refresh(CarlaHostHandle handle, bool external);
 
 /*!
  * Start playback of the engine transport.
  */
-CARLA_EXPORT void carla_transport_play(void);
+CARLA_EXPORT void carla_transport_play(CarlaHostHandle handle);
 
 /*!
  * Pause the engine transport.
  */
-CARLA_EXPORT void carla_transport_pause(void);
+CARLA_EXPORT void carla_transport_pause(CarlaHostHandle handle);
 
 /*!
  * Set the engine transport bpm.
  */
-CARLA_EXPORT void carla_transport_bpm(double bpm);
+CARLA_EXPORT void carla_transport_bpm(CarlaHostHandle handle, double bpm);
 
 /*!
  * Relocate the engine transport to a specific frame.
  */
-CARLA_EXPORT void carla_transport_relocate(uint64_t frame);
+CARLA_EXPORT void carla_transport_relocate(CarlaHostHandle handle, uint64_t frame);
 
 /*!
  * Get the current transport frame.
  */
-CARLA_EXPORT uint64_t carla_get_current_transport_frame(void);
+CARLA_EXPORT uint64_t carla_get_current_transport_frame(CarlaHostHandle handle);
 
 /*!
  * Get the engine transport information.
  */
-CARLA_EXPORT const CarlaTransportInfo* carla_get_transport_info(void);
+CARLA_EXPORT const CarlaTransportInfo* carla_get_transport_info(CarlaHostHandle handle);
 #endif
 
 /*!
  * Current number of plugins loaded.
  */
-CARLA_EXPORT uint32_t carla_get_current_plugin_count(void);
+CARLA_EXPORT uint32_t carla_get_current_plugin_count(CarlaHostHandle handle);
 
 /*!
  * Maximum number of loadable plugins allowed.
  * Returns 0 if engine is not started.
  */
-CARLA_EXPORT uint32_t carla_get_max_plugin_number(void);
+CARLA_EXPORT uint32_t carla_get_max_plugin_number(CarlaHostHandle handle);
 
 /*!
  * Add a new plugin.
@@ -597,7 +628,8 @@ CARLA_EXPORT uint32_t carla_get_max_plugin_number(void);
  * @param extraPtr Extra pointer, defined per plugin type
  * @param options  Initial plugin options
  */
-CARLA_EXPORT bool carla_add_plugin(BinaryType btype, PluginType ptype,
+CARLA_EXPORT bool carla_add_plugin(CarlaHostHandle handle,
+                                   BinaryType btype, PluginType ptype,
                                    const char* filename, const char* name, const char* label, int64_t uniqueId,
                                    const void* extraPtr, uint options);
 
@@ -605,12 +637,12 @@ CARLA_EXPORT bool carla_add_plugin(BinaryType btype, PluginType ptype,
  * Remove one plugin.
  * @param pluginId Plugin to remove.
  */
-CARLA_EXPORT bool carla_remove_plugin(uint pluginId);
+CARLA_EXPORT bool carla_remove_plugin(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Remove all plugins.
  */
-CARLA_EXPORT bool carla_remove_all_plugins(void);
+CARLA_EXPORT bool carla_remove_all_plugins(CarlaHostHandle handle);
 
 #ifndef BUILD_BRIDGE
 /*!
@@ -619,13 +651,13 @@ CARLA_EXPORT bool carla_remove_all_plugins(void);
  * @param pluginId Plugin to rename
  * @param newName  New plugin name
  */
-CARLA_EXPORT bool carla_rename_plugin(uint pluginId, const char* newName);
+CARLA_EXPORT bool carla_rename_plugin(CarlaHostHandle handle, uint pluginId, const char* newName);
 
 /*!
  * Clone a plugin.
  * @param pluginId Plugin to clone
  */
-CARLA_EXPORT bool carla_clone_plugin(uint pluginId);
+CARLA_EXPORT bool carla_clone_plugin(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Prepare replace of a plugin.
@@ -633,14 +665,14 @@ CARLA_EXPORT bool carla_clone_plugin(uint pluginId);
  * @param pluginId Plugin to replace
  * @note This function requires carla_add_plugin() to be called afterwards *as soon as possible*.
  */
-CARLA_EXPORT bool carla_replace_plugin(uint pluginId);
+CARLA_EXPORT bool carla_replace_plugin(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Switch two plugins positions.
  * @param pluginIdA Plugin A
  * @param pluginIdB Plugin B
  */
-CARLA_EXPORT bool carla_switch_plugins(uint pluginIdA, uint pluginIdB);
+CARLA_EXPORT bool carla_switch_plugins(CarlaHostHandle handle, uint pluginIdA, uint pluginIdB);
 #endif
 
 /*!
@@ -649,7 +681,7 @@ CARLA_EXPORT bool carla_switch_plugins(uint pluginIdA, uint pluginIdB);
  * @param filename Path to plugin state
  * @see carla_save_plugin_state()
  */
-CARLA_EXPORT bool carla_load_plugin_state(uint pluginId, const char* filename);
+CARLA_EXPORT bool carla_load_plugin_state(CarlaHostHandle handle, uint pluginId, const char* filename);
 
 /*!
  * Save a plugin state.
@@ -657,38 +689,38 @@ CARLA_EXPORT bool carla_load_plugin_state(uint pluginId, const char* filename);
  * @param filename Path to plugin state
  * @see carla_load_plugin_state()
  */
-CARLA_EXPORT bool carla_save_plugin_state(uint pluginId, const char* filename);
+CARLA_EXPORT bool carla_save_plugin_state(CarlaHostHandle handle, uint pluginId, const char* filename);
 
 /*!
  * Export plugin as LV2.
  * @param pluginId Plugin
  * @param lv2path Path to lv2 plugin folder
  */
-CARLA_EXPORT bool carla_export_plugin_lv2(uint pluginId, const char* lv2path);
+CARLA_EXPORT bool carla_export_plugin_lv2(CarlaHostHandle handle, uint pluginId, const char* lv2path);
 
 /*!
  * Get information from a plugin.
  * @param pluginId Plugin
  */
-CARLA_EXPORT const CarlaPluginInfo* carla_get_plugin_info(uint pluginId);
+CARLA_EXPORT const CarlaPluginInfo* carla_get_plugin_info(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Get audio port count information from a plugin.
  * @param pluginId Plugin
  */
-CARLA_EXPORT const CarlaPortCountInfo* carla_get_audio_port_count_info(uint pluginId);
+CARLA_EXPORT const CarlaPortCountInfo* carla_get_audio_port_count_info(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Get MIDI port count information from a plugin.
  * @param pluginId Plugin
  */
-CARLA_EXPORT const CarlaPortCountInfo* carla_get_midi_port_count_info(uint pluginId);
+CARLA_EXPORT const CarlaPortCountInfo* carla_get_midi_port_count_info(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Get parameter count information from a plugin.
  * @param pluginId Plugin
  */
-CARLA_EXPORT const CarlaPortCountInfo* carla_get_parameter_count_info(uint pluginId);
+CARLA_EXPORT const CarlaPortCountInfo* carla_get_parameter_count_info(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Get parameter information from a plugin.
@@ -696,7 +728,9 @@ CARLA_EXPORT const CarlaPortCountInfo* carla_get_parameter_count_info(uint plugi
  * @param parameterId Parameter index
  * @see carla_get_parameter_count()
  */
-CARLA_EXPORT const CarlaParameterInfo* carla_get_parameter_info(uint pluginId, uint32_t parameterId);
+CARLA_EXPORT const CarlaParameterInfo* carla_get_parameter_info(CarlaHostHandle handle,
+                                                                uint pluginId,
+                                                                uint32_t parameterId);
 
 /*!
  * Get parameter scale point information from a plugin.
@@ -705,7 +739,10 @@ CARLA_EXPORT const CarlaParameterInfo* carla_get_parameter_info(uint pluginId, u
  * @param scalePointId Parameter scale-point index
  * @see CarlaParameterInfo::scalePointCount
  */
-CARLA_EXPORT const CarlaScalePointInfo* carla_get_parameter_scalepoint_info(uint pluginId, uint32_t parameterId, uint32_t scalePointId);
+CARLA_EXPORT const CarlaScalePointInfo* carla_get_parameter_scalepoint_info(CarlaHostHandle handle,
+                                                                            uint pluginId,
+                                                                            uint32_t parameterId,
+                                                                            uint32_t scalePointId);
 
 /*!
  * Get a plugin's parameter data.
@@ -713,7 +750,9 @@ CARLA_EXPORT const CarlaScalePointInfo* carla_get_parameter_scalepoint_info(uint
  * @param parameterId Parameter index
  * @see carla_get_parameter_count()
  */
-CARLA_EXPORT const ParameterData* carla_get_parameter_data(uint pluginId, uint32_t parameterId);
+CARLA_EXPORT const ParameterData* carla_get_parameter_data(CarlaHostHandle handle,
+                                                           uint pluginId,
+                                                           uint32_t parameterId);
 
 /*!
  * Get a plugin's parameter ranges.
@@ -721,7 +760,9 @@ CARLA_EXPORT const ParameterData* carla_get_parameter_data(uint pluginId, uint32
  * @param parameterId Parameter index
  * @see carla_get_parameter_count()
  */
-CARLA_EXPORT const ParameterRanges* carla_get_parameter_ranges(uint pluginId, uint32_t parameterId);
+CARLA_EXPORT const ParameterRanges* carla_get_parameter_ranges(CarlaHostHandle handle,
+                                                               uint pluginId,
+                                                               uint32_t parameterId);
 
 /*!
  * Get a plugin's MIDI program data.
@@ -729,7 +770,9 @@ CARLA_EXPORT const ParameterRanges* carla_get_parameter_ranges(uint pluginId, ui
  * @param midiProgramId MIDI Program index
  * @see carla_get_midi_program_count()
  */
-CARLA_EXPORT const MidiProgramData* carla_get_midi_program_data(uint pluginId, uint32_t midiProgramId);
+CARLA_EXPORT const MidiProgramData* carla_get_midi_program_data(CarlaHostHandle handle,
+                                                                uint pluginId,
+                                                                uint32_t midiProgramId);
 
 /*!
  * Get a plugin's custom data, using index.
@@ -737,7 +780,7 @@ CARLA_EXPORT const MidiProgramData* carla_get_midi_program_data(uint pluginId, u
  * @param customDataId Custom data index
  * @see carla_get_custom_data_count()
  */
-CARLA_EXPORT const CustomData* carla_get_custom_data(uint pluginId, uint32_t customDataId);
+CARLA_EXPORT const CustomData* carla_get_custom_data(CarlaHostHandle handle, uint pluginId, uint32_t customDataId);
 
 /*!
  * Get a plugin's custom data value, using type and key.
@@ -746,41 +789,44 @@ CARLA_EXPORT const CustomData* carla_get_custom_data(uint pluginId, uint32_t cus
  * @param key      Custom data key
  * @see carla_get_custom_data_count()
  */
-CARLA_EXPORT const char* carla_get_custom_data_value(uint pluginId, const char* type, const char* key);
+CARLA_EXPORT const char* carla_get_custom_data_value(CarlaHostHandle handle,
+                                                     uint pluginId,
+                                                     const char* type,
+                                                     const char* key);
 
 /*!
  * Get a plugin's chunk data.
  * @param pluginId Plugin
  * @see PLUGIN_OPTION_USE_CHUNKS and carla_set_chunk_data()
  */
-CARLA_EXPORT const char* carla_get_chunk_data(uint pluginId);
+CARLA_EXPORT const char* carla_get_chunk_data(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Get how many parameters a plugin has.
  * @param pluginId Plugin
  */
-CARLA_EXPORT uint32_t carla_get_parameter_count(uint pluginId);
+CARLA_EXPORT uint32_t carla_get_parameter_count(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Get how many programs a plugin has.
  * @param pluginId Plugin
  * @see carla_get_program_name()
  */
-CARLA_EXPORT uint32_t carla_get_program_count(uint pluginId);
+CARLA_EXPORT uint32_t carla_get_program_count(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Get how many MIDI programs a plugin has.
  * @param pluginId Plugin
  * @see carla_get_midi_program_name() and carla_get_midi_program_data()
  */
-CARLA_EXPORT uint32_t carla_get_midi_program_count(uint pluginId);
+CARLA_EXPORT uint32_t carla_get_midi_program_count(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Get how many custom data sets a plugin has.
  * @param pluginId Plugin
  * @see carla_get_custom_data()
  */
-CARLA_EXPORT uint32_t carla_get_custom_data_count(uint pluginId);
+CARLA_EXPORT uint32_t carla_get_custom_data_count(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Get a plugin's parameter text (custom display of internal values).
@@ -788,7 +834,7 @@ CARLA_EXPORT uint32_t carla_get_custom_data_count(uint pluginId);
  * @param parameterId Parameter index
  * @see PARAMETER_USES_CUSTOM_TEXT
  */
-CARLA_EXPORT const char* carla_get_parameter_text(uint pluginId, uint32_t parameterId);
+CARLA_EXPORT const char* carla_get_parameter_text(CarlaHostHandle handle, uint pluginId, uint32_t parameterId);
 
 /*!
  * Get a plugin's program name.
@@ -796,7 +842,7 @@ CARLA_EXPORT const char* carla_get_parameter_text(uint pluginId, uint32_t parame
  * @param programId Program index
  * @see carla_get_program_count()
  */
-CARLA_EXPORT const char* carla_get_program_name(uint pluginId, uint32_t programId);
+CARLA_EXPORT const char* carla_get_program_name(CarlaHostHandle handle, uint pluginId, uint32_t programId);
 
 /*!
  * Get a plugin's MIDI program name.
@@ -804,40 +850,40 @@ CARLA_EXPORT const char* carla_get_program_name(uint pluginId, uint32_t programI
  * @param midiProgramId MIDI Program index
  * @see carla_get_midi_program_count()
  */
-CARLA_EXPORT const char* carla_get_midi_program_name(uint pluginId, uint32_t midiProgramId);
+CARLA_EXPORT const char* carla_get_midi_program_name(CarlaHostHandle handle, uint pluginId, uint32_t midiProgramId);
 
 /*!
  * Get a plugin's real name.
  * This is the name the plugin uses to identify itself; may not be unique.
  * @param pluginId Plugin
  */
-CARLA_EXPORT const char* carla_get_real_plugin_name(uint pluginId);
+CARLA_EXPORT const char* carla_get_real_plugin_name(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Get a plugin's program index.
  * @param pluginId Plugin
  */
-CARLA_EXPORT int32_t carla_get_current_program_index(uint pluginId);
+CARLA_EXPORT int32_t carla_get_current_program_index(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Get a plugin's midi program index.
  * @param pluginId Plugin
  */
-CARLA_EXPORT int32_t carla_get_current_midi_program_index(uint pluginId);
+CARLA_EXPORT int32_t carla_get_current_midi_program_index(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Get a plugin's default parameter value.
  * @param pluginId    Plugin
  * @param parameterId Parameter index
  */
-CARLA_EXPORT float carla_get_default_parameter_value(uint pluginId, uint32_t parameterId);
+CARLA_EXPORT float carla_get_default_parameter_value(CarlaHostHandle handle, uint pluginId, uint32_t parameterId);
 
 /*!
  * Get a plugin's current parameter value.
  * @param pluginId    Plugin
  * @param parameterId Parameter index
  */
-CARLA_EXPORT float carla_get_current_parameter_value(uint pluginId, uint32_t parameterId);
+CARLA_EXPORT float carla_get_current_parameter_value(CarlaHostHandle handle, uint pluginId, uint32_t parameterId);
 
 /*!
  * Get a plugin's internal parameter value.
@@ -845,40 +891,43 @@ CARLA_EXPORT float carla_get_current_parameter_value(uint pluginId, uint32_t par
  * @param parameterId Parameter index, maybe be negative
  * @see InternalParameterIndex
  */
-CARLA_EXPORT float carla_get_internal_parameter_value(uint pluginId, int32_t parameterId);
+CARLA_EXPORT float carla_get_internal_parameter_value(CarlaHostHandle handle, uint pluginId, int32_t parameterId);
 
 /*!
  * Get a plugin's peak values.
  * @param pluginId Plugin
  */
-CARLA_EXPORT const float* carla_get_peak_values(uint pluginId);
+CARLA_EXPORT const float* carla_get_peak_values(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Get a plugin's input peak value.
  * @param pluginId Plugin
  * @param isLeft   Wherever to get the left/mono value, otherwise right.
  */
-CARLA_EXPORT float carla_get_input_peak_value(uint pluginId, bool isLeft);
+CARLA_EXPORT float carla_get_input_peak_value(CarlaHostHandle handle, uint pluginId, bool isLeft);
 
 /*!
  * Get a plugin's output peak value.
  * @param pluginId Plugin
  * @param isLeft   Wherever to get the left/mono value, otherwise right.
  */
-CARLA_EXPORT float carla_get_output_peak_value(uint pluginId, bool isLeft);
+CARLA_EXPORT float carla_get_output_peak_value(CarlaHostHandle handle, uint pluginId, bool isLeft);
 
 /*!
  * Render a plugin's inline display.
  * @param pluginId Plugin
  */
-CARLA_EXPORT const CarlaInlineDisplayImageSurface* carla_render_inline_display(uint pluginId, uint32_t width, uint32_t height);
+CARLA_EXPORT const CarlaInlineDisplayImageSurface* carla_render_inline_display(CarlaHostHandle handle,
+                                                                               uint pluginId,
+                                                                               uint32_t width,
+                                                                               uint32_t height);
 
 /*!
  * Enable or disable a plugin.
  * @param pluginId Plugin
  * @param onOff    New active state
  */
-CARLA_EXPORT void carla_set_active(uint pluginId, bool onOff);
+CARLA_EXPORT void carla_set_active(CarlaHostHandle handle, uint pluginId, bool onOff);
 
 #ifndef BUILD_BRIDGE
 /*!
@@ -886,42 +935,42 @@ CARLA_EXPORT void carla_set_active(uint pluginId, bool onOff);
  * @param pluginId Plugin
  * @param value    New dry/wet value
  */
-CARLA_EXPORT void carla_set_drywet(uint pluginId, float value);
+CARLA_EXPORT void carla_set_drywet(CarlaHostHandle handle, uint pluginId, float value);
 
 /*!
  * Change a plugin's internal volume.
  * @param pluginId Plugin
  * @param value    New volume
  */
-CARLA_EXPORT void carla_set_volume(uint pluginId, float value);
+CARLA_EXPORT void carla_set_volume(CarlaHostHandle handle, uint pluginId, float value);
 
 /*!
  * Change a plugin's internal stereo balance, left channel.
  * @param pluginId Plugin
  * @param value    New value
  */
-CARLA_EXPORT void carla_set_balance_left(uint pluginId, float value);
+CARLA_EXPORT void carla_set_balance_left(CarlaHostHandle handle, uint pluginId, float value);
 
 /*!
  * Change a plugin's internal stereo balance, right channel.
  * @param pluginId Plugin
  * @param value    New value
  */
-CARLA_EXPORT void carla_set_balance_right(uint pluginId, float value);
+CARLA_EXPORT void carla_set_balance_right(CarlaHostHandle handle, uint pluginId, float value);
 
 /*!
  * Change a plugin's internal mono panning value.
  * @param pluginId Plugin
  * @param value    New value
  */
-CARLA_EXPORT void carla_set_panning(uint pluginId, float value);
+CARLA_EXPORT void carla_set_panning(CarlaHostHandle handle, uint pluginId, float value);
 
 /*!
  * Change a plugin's internal control channel.
  * @param pluginId Plugin
  * @param channel  New channel
  */
-CARLA_EXPORT void carla_set_ctrl_channel(uint pluginId, int8_t channel);
+CARLA_EXPORT void carla_set_ctrl_channel(CarlaHostHandle handle, uint pluginId, int8_t channel);
 #endif
 
 /*!
@@ -930,7 +979,7 @@ CARLA_EXPORT void carla_set_ctrl_channel(uint pluginId, int8_t channel);
  * @param option   An option from PluginOptions
  * @param yesNo    New enabled state
  */
-CARLA_EXPORT void carla_set_option(uint pluginId, uint option, bool yesNo);
+CARLA_EXPORT void carla_set_option(CarlaHostHandle handle, uint pluginId, uint option, bool yesNo);
 
 /*!
  * Change a plugin's parameter value.
@@ -938,7 +987,7 @@ CARLA_EXPORT void carla_set_option(uint pluginId, uint option, bool yesNo);
  * @param parameterId Parameter index
  * @param value       New value
  */
-CARLA_EXPORT void carla_set_parameter_value(uint pluginId, uint32_t parameterId, float value);
+CARLA_EXPORT void carla_set_parameter_value(CarlaHostHandle handle, uint pluginId, uint32_t parameterId, float value);
 
 #ifndef BUILD_BRIDGE
 /*!
@@ -947,7 +996,10 @@ CARLA_EXPORT void carla_set_parameter_value(uint pluginId, uint32_t parameterId,
  * @param parameterId Parameter index
  * @param channel     New MIDI channel
  */
-CARLA_EXPORT void carla_set_parameter_midi_channel(uint pluginId, uint32_t parameterId, uint8_t channel);
+CARLA_EXPORT void carla_set_parameter_midi_channel(CarlaHostHandle handle,
+                                                   uint pluginId,
+                                                   uint32_t parameterId,
+                                                   uint8_t channel);
 
 /*!
  * Change a plugin's parameter mapped control index.
@@ -955,7 +1007,10 @@ CARLA_EXPORT void carla_set_parameter_midi_channel(uint pluginId, uint32_t param
  * @param parameterId Parameter index
  * @param cc          New control index
  */
-CARLA_EXPORT void carla_set_parameter_mapped_control_index(uint pluginId, uint32_t parameterId, int16_t index);
+CARLA_EXPORT void carla_set_parameter_mapped_control_index(CarlaHostHandle handle,
+                                                           uint pluginId,
+                                                           uint32_t parameterId,
+                                                           int16_t index);
 
 /*!
  * Change a plugin's parameter mapped range.
@@ -964,7 +1019,10 @@ CARLA_EXPORT void carla_set_parameter_mapped_control_index(uint pluginId, uint32
  * @param minimum     New mapped minimum
  * @param maximum     New mapped maximum
  */
-CARLA_EXPORT void carla_set_parameter_mapped_range(uint pluginId, uint32_t parameterId, float minimum, float maximum);
+CARLA_EXPORT void carla_set_parameter_mapped_range(CarlaHostHandle handle,
+                                                   uint pluginId,
+                                                   uint32_t parameterId,
+                                                   float minimum, float maximum);
 
 /*!
  * Change a plugin's parameter in drag/touch mode state.
@@ -973,7 +1031,10 @@ CARLA_EXPORT void carla_set_parameter_mapped_range(uint pluginId, uint32_t param
  * @param parameterId Parameter index
  * @param touch       New state
  */
-CARLA_EXPORT void carla_set_parameter_touch(uint pluginId, uint32_t parameterId, bool touch);
+CARLA_EXPORT void carla_set_parameter_touch(CarlaHostHandle handle,
+                                            uint pluginId,
+                                            uint32_t parameterId,
+                                            bool touch);
 #endif
 
 /*!
@@ -981,14 +1042,14 @@ CARLA_EXPORT void carla_set_parameter_touch(uint pluginId, uint32_t parameterId,
  * @param pluginId  Plugin
  * @param programId New program
  */
-CARLA_EXPORT void carla_set_program(uint pluginId, uint32_t programId);
+CARLA_EXPORT void carla_set_program(CarlaHostHandle handle, uint pluginId, uint32_t programId);
 
 /*!
  * Change a plugin's current MIDI program.
  * @param pluginId      Plugin
  * @param midiProgramId New value
  */
-CARLA_EXPORT void carla_set_midi_program(uint pluginId, uint32_t midiProgramId);
+CARLA_EXPORT void carla_set_midi_program(CarlaHostHandle handle, uint pluginId, uint32_t midiProgramId);
 
 /*!
  * Set a plugin's custom data set.
@@ -998,7 +1059,9 @@ CARLA_EXPORT void carla_set_midi_program(uint pluginId, uint32_t midiProgramId);
  * @param value    New value
  * @see CustomDataTypes and CustomDataKeys
  */
-CARLA_EXPORT void carla_set_custom_data(uint pluginId, const char* type, const char* key, const char* value);
+CARLA_EXPORT void carla_set_custom_data(CarlaHostHandle handle,
+                                        uint pluginId,
+                                        const char* type, const char* key, const char* value);
 
 /*!
  * Set a plugin's chunk data.
@@ -1006,26 +1069,26 @@ CARLA_EXPORT void carla_set_custom_data(uint pluginId, const char* type, const c
  * @param chunkData New chunk data
  * @see PLUGIN_OPTION_USE_CHUNKS and carla_get_chunk_data()
  */
-CARLA_EXPORT void carla_set_chunk_data(uint pluginId, const char* chunkData);
+CARLA_EXPORT void carla_set_chunk_data(CarlaHostHandle handle, uint pluginId, const char* chunkData);
 
 /*!
  * Tell a plugin to prepare for save.
  * This should be called before saving custom data sets.
  * @param pluginId Plugin
  */
-CARLA_EXPORT void carla_prepare_for_save(uint pluginId);
+CARLA_EXPORT void carla_prepare_for_save(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Reset all plugin's parameters.
  * @param pluginId Plugin
  */
-CARLA_EXPORT void carla_reset_parameters(uint pluginId);
+CARLA_EXPORT void carla_reset_parameters(CarlaHostHandle handle, uint pluginId);
 
 /*!
  * Randomize all plugin's parameters.
  * @param pluginId Plugin
  */
-CARLA_EXPORT void carla_randomize_parameters(uint pluginId);
+CARLA_EXPORT void carla_randomize_parameters(CarlaHostHandle handle, uint pluginId);
 
 #ifndef BUILD_BRIDGE
 /*!
@@ -1036,41 +1099,89 @@ CARLA_EXPORT void carla_randomize_parameters(uint pluginId);
  * @param note     Note pitch
  * @param velocity Note velocity
  */
-CARLA_EXPORT void carla_send_midi_note(uint pluginId, uint8_t channel, uint8_t note, uint8_t velocity);
+CARLA_EXPORT void carla_send_midi_note(CarlaHostHandle handle,
+                                       uint pluginId,
+                                       uint8_t channel, uint8_t note, uint8_t velocity);
 #endif
 
+/*!
+ * Set a custom title for the plugin UI window created by Carla.
+ */
+CARLA_EXPORT void carla_set_custom_ui_title(CarlaHostHandle handle, uint pluginId, const char* title);
+
 /*!
  * Tell a plugin to show its own custom UI.
  * @param pluginId Plugin
  * @param yesNo    New UI state, visible or not
  * @see PLUGIN_HAS_CUSTOM_UI
  */
-CARLA_EXPORT void carla_show_custom_ui(uint pluginId, bool yesNo);
+CARLA_EXPORT void carla_show_custom_ui(CarlaHostHandle handle, uint pluginId, bool yesNo);
+
+/*!
+  * Embed the plugin's custom UI to the system pointer @a ptr.
+  * This function is always called from the main thread.
+  * @note This is very experimental and subject to change at this point
+  */
+CARLA_EXPORT void* carla_embed_custom_ui(CarlaHostHandle handle, uint pluginId, void* ptr);
 
 /*!
  * Get the current engine buffer size.
  */
-CARLA_EXPORT uint32_t carla_get_buffer_size(void);
+CARLA_EXPORT uint32_t carla_get_buffer_size(CarlaHostHandle handle);
 
 /*!
  * Get the current engine sample rate.
  */
-CARLA_EXPORT double carla_get_sample_rate(void);
+CARLA_EXPORT double carla_get_sample_rate(CarlaHostHandle handle);
 
 /*!
  * Get the last error.
  */
-CARLA_EXPORT const char* carla_get_last_error(void);
+CARLA_EXPORT const char* carla_get_last_error(CarlaHostHandle handle);
 
 /*!
  * Get the current engine OSC URL (TCP).
  */
-CARLA_EXPORT const char* carla_get_host_osc_url_tcp(void);
+CARLA_EXPORT const char* carla_get_host_osc_url_tcp(CarlaHostHandle handle);
 
 /*!
  * Get the current engine OSC URL (UDP).
  */
-CARLA_EXPORT const char* carla_get_host_osc_url_udp(void);
+CARLA_EXPORT const char* carla_get_host_osc_url_udp(CarlaHostHandle handle);
+
+/*!
+ * Initialize NSM (that is, announce ourselves to it).
+ * Must be called as early as possible in the program's lifecycle.
+ * Returns true if NSM is available and initialized correctly.
+ */
+CARLA_EXPORT bool carla_nsm_init(CarlaHostHandle handle, uint64_t pid, const char* executableName);
+
+/*!
+ * Respond to an NSM callback.
+ */
+CARLA_EXPORT void carla_nsm_ready(CarlaHostHandle handle, NsmCallbackOpcode opcode);
+
+#ifndef CARLA_UTILS_H_INCLUDED
+/*!
+ * Get the complete license text of used third-party code and features.
+ * Returned string is in basic html format.
+ */
+CARLA_EXPORT const char* carla_get_complete_license_text(void);
+
+/*!
+ * Get the juce version used in the current Carla build.
+ */
+CARLA_EXPORT const char* carla_get_juce_version(void);
+
+/*!
+ * Get the list of supported file extensions in carla_load_file().
+ */
+CARLA_EXPORT const char* const* carla_get_supported_file_extensions(void);
+
+/*!
+ * Get the list of supported features in the current Carla build.
+ */
+CARLA_EXPORT const char* const* carla_get_supported_features(void);
 
 /*!
  * Get the absolute filename of this carla library.
@@ -1081,18 +1192,7 @@ CARLA_EXPORT const char* carla_get_library_filename(void);
  * Get the folder where this carla library resides.
  */
 CARLA_EXPORT const char* carla_get_library_folder(void);
-
-/*!
- * Initialize NSM (that is, announce ourselves to it).
- * Must be called as early as possible in the program's lifecycle.
- * Returns true if NSM is available and initialized correctly.
- */
-CARLA_EXPORT bool carla_nsm_init(uint64_t pid, const char* executableName);
-
-/*!
- * Respond to an NSM callback.
- */
-CARLA_EXPORT void carla_nsm_ready(NsmCallbackOpcode opcode);
+#endif
 
 /** @} */
 
