@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Parameter SpinBox, a custom Qt4 widget
-# Copyright (C) 2011-2014 Filipe Coelho <falktx@falktx.com>
+# Parameter SpinBox, a custom Qt widget
+# Copyright (C) 2011-2019 Filipe Coelho <falktx@falktx.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -16,30 +16,23 @@
 #
 # For a full copy of the GNU General Public License see the doc/GPL.txt file.
 
-# ------------------------------------------------------------------------------------------------------------
-# Imports (Config)
-
-from carla_config import *
-
 # ------------------------------------------------------------------------------------------------------------
 # Imports (Global)
 
 from math import isnan, modf
 from random import random
 
-if config_UseQt5:
-    from PyQt5.QtCore import pyqtSignal, pyqtSlot, Qt, QTimer
-    from PyQt5.QtGui import QCursor
-    from PyQt5.QtWidgets import QAbstractSpinBox, QApplication, QComboBox, QDialog, QMenu, QProgressBar
-else:
-    from PyQt4.QtCore import pyqtSignal, pyqtSlot, Qt, QTimer
-    from PyQt4.QtGui import QAbstractSpinBox, QApplication, QComboBox, QCursor, QDialog, QMenu, QProgressBar
+from PyQt5.QtCore import pyqtSignal, pyqtSlot, Qt, QTimer
+from PyQt5.QtGui import QCursor, QPalette
+from PyQt5.QtWidgets import QAbstractSpinBox, QApplication, QComboBox, QDialog, QMenu, QProgressBar
 
 # ------------------------------------------------------------------------------------------------------------
 # Imports (Custom)
 
 import ui_inputdialog_value
 
+from carla_shared import countDecimalPoints, MACOS
+
 # ------------------------------------------------------------------------------------------------------------
 # Get a fixed value within min/max bounds
 
@@ -59,31 +52,22 @@ def geFixedValue(name, value, minimum, maximum):
 # Custom InputDialog with Scale Points support
 
 class CustomInputDialog(QDialog):
-    def __init__(self, parent, label, current, minimum, maximum, step, stepSmall, scalePoints):
+    def __init__(self, parent, label, current, minimum, maximum, step, stepSmall, scalePoints, prefix, suffix):
         QDialog.__init__(self, parent)
         self.ui = ui_inputdialog_value.Ui_Dialog()
         self.ui.setupUi(self)
 
-        # calculate num decimals from stepSmall
-        if stepSmall >= 1.0:
-            decimals = 0
-        elif step >= 1.0:
-            decimals = 2
-        else:
-            decfrac, decwhole = modf(stepSmall)
-
-            if "000" in str(decfrac):
-                decfrac = round(decfrac, str(decfrac).find("000"))
-            else:
-                decfrac = round(decfrac, 12)
-
-            decimals = abs(len(str(decfrac))-len(str(decwhole))-1)
-
+        decimals = countDecimalPoints(step, stepSmall)
         self.ui.label.setText(label)
         self.ui.doubleSpinBox.setDecimals(decimals)
         self.ui.doubleSpinBox.setRange(minimum, maximum)
         self.ui.doubleSpinBox.setSingleStep(step)
         self.ui.doubleSpinBox.setValue(current)
+        self.ui.doubleSpinBox.setPrefix(prefix)
+        self.ui.doubleSpinBox.setSuffix(suffix)
+
+        if MACOS:
+            self.setWindowModality(Qt.WindowModal)
 
         if not scalePoints:
             self.ui.groupBox.setVisible(False)
@@ -91,13 +75,14 @@ class CustomInputDialog(QDialog):
         else:
             text = "<table>"
             for scalePoint in scalePoints:
-                text += "<tr><td align='right'>%f</td><td align='left'> - %s</td></tr>" % (scalePoint['value'], scalePoint['label'])
+                valuestr = ("%i" if decimals == 0 else "%f") % scalePoint['value']
+                text += "<tr><td align='right'>%s</td><td align='left'> - %s</td></tr>" % (valuestr, scalePoint['label'])
             text += "</table>"
             self.ui.textBrowser.setText(text)
             self.resize(200, 300)
 
         self.fRetValue = current
-
+        self.adjustSize()
         self.accepted.connect(self.slot_setReturnValue)
 
     def returnValue(self):
@@ -116,6 +101,7 @@ class CustomInputDialog(QDialog):
 
 class ParamProgressBar(QProgressBar):
     # signals
+    dragStateChanged = pyqtSignal(bool)
     valueChanged = pyqtSignal(float)
 
     def __init__(self, parent):
@@ -133,9 +119,9 @@ class ParamProgressBar(QProgressBar):
         self.fLastPaintedValue   = None
         self.fCurrentPaintedText = ""
 
-        self.fLabel = ""
         self.fName  = ""
-        self.fPreLabel = " "
+        self.fLabelPrefix = ""
+        self.fLabelSuffix = ""
         self.fTextCall  = None
         self.fValueCall = None
 
@@ -153,7 +139,7 @@ class ParamProgressBar(QProgressBar):
         self.fMaximum = value
 
     def setValue(self, value):
-        if self.fRealValue == value and self.fInitiated:
+        if (self.fRealValue == value or isnan(value)) and self.fInitiated:
             return False
 
         self.fInitiated = True
@@ -161,7 +147,16 @@ class ParamProgressBar(QProgressBar):
         div = float(self.fMaximum - self.fMinimum)
 
         if div == 0.0:
-            print("Parameter '%s' division by 0 prevented (value:%f, min:%f, max:%f)" % (self.fName, value, self.fMaximum, self.fMinimum))
+            print("Parameter '%s' division by 0 prevented (value:%f, min:%f, max:%f)" % (self.fName,
+                                                                                         value,
+                                                                                         self.fMaximum,
+                                                                                         self.fMinimum))
+            vper = 1.0
+        elif isnan(value):
+            print("Parameter '%s' is NaN (value:%f, min:%f, max:%f)" % (self.fName,
+                                                                        value,
+                                                                        self.fMaximum,
+                                                                        self.fMinimum))
             vper = 1.0
         else:
             vper = float(value - self.fMinimum) / div
@@ -177,12 +172,9 @@ class ParamProgressBar(QProgressBar):
         QProgressBar.setValue(self, int(vper * 10000))
         return True
 
-    def setLabel(self, label):
-        self.fLabel = label.strip()
-
-        if self.fLabel == "(coef)":
-            self.fLabel = ""
-            self.fPreLabel = "*"
+    def setSuffixes(self, prefix, suffix):
+        self.fLabelPrefix = prefix
+        self.fLabelSuffix = suffix
 
         # force refresh of text value
         self.fLastPaintedValue = None
@@ -226,6 +218,7 @@ class ParamProgressBar(QProgressBar):
         if event.button() == Qt.LeftButton:
             self.handleMouseEventPos(event.pos())
             self.fLeftClickDown = True
+            self.dragStateChanged.emit(True)
         else:
             self.fLeftClickDown = False
 
@@ -245,6 +238,7 @@ class ParamProgressBar(QProgressBar):
             return
 
         self.fLeftClickDown = False
+        self.dragStateChanged.emit(False)
         QProgressBar.mouseReleaseEvent(self, event)
 
     def paintEvent(self, event):
@@ -252,13 +246,13 @@ class ParamProgressBar(QProgressBar):
             if self.fLastPaintedValue != self.fRealValue:
                 self.fLastPaintedValue   = self.fRealValue
                 self.fCurrentPaintedText = self.fTextCall()
-            self.setFormat("%s %s %s" % (self.fPreLabel, self.fCurrentPaintedText, self.fLabel))
+            self.setFormat("%s%s%s" % (self.fLabelPrefix, self.fCurrentPaintedText, self.fLabelSuffix))
 
         elif self.fIsInteger:
-            self.setFormat("%s %i %s" % (self.fPreLabel, int(self.fRealValue), self.fLabel))
+            self.setFormat("%s%i%s" % (self.fLabelPrefix, int(self.fRealValue), self.fLabelSuffix))
 
         else:
-            self.setFormat("%s %f %s" % (self.fPreLabel, self.fRealValue, self.fLabel))
+            self.setFormat("%s%f%s" % (self.fLabelPrefix, self.fRealValue, self.fLabelSuffix))
 
         QProgressBar.paintEvent(self, event)
 
@@ -273,6 +267,8 @@ class ParamSpinBox(QAbstractSpinBox):
         QAbstractSpinBox.__init__(self, parent)
 
         self.fName = ""
+        self.fLabelPrefix = ""
+        self.fLabelSuffix = ""
 
         self.fMinimum = 0.0
         self.fMaximum = 1.0
@@ -291,6 +287,10 @@ class ParamSpinBox(QAbstractSpinBox):
         self.fBar.setContextMenuPolicy(Qt.NoContextMenu)
         #self.fBar.show()
 
+        barPalette = self.fBar.palette()
+        barPalette.setColor(QPalette.Window, Qt.transparent)
+        self.fBar.setPalette(barPalette)
+
         self.fBox = None
 
         self.lineEdit().hide()
@@ -298,6 +298,8 @@ class ParamSpinBox(QAbstractSpinBox):
         self.customContextMenuRequested.connect(self.slot_showCustomMenu)
         self.fBar.valueChanged.connect(self.slot_progressBarValueChanged)
 
+        self.dragStateChanged = self.fBar.dragStateChanged
+
         QTimer.singleShot(0, self.slot_updateProgressBarGeometry)
 
     def setDefault(self, value):
@@ -365,7 +367,18 @@ class ParamSpinBox(QAbstractSpinBox):
             self.fStepLarge = value
 
     def setLabel(self, label):
-        self.fBar.setLabel(label)
+        prefix = ""
+        suffix = label.strip()
+
+        if suffix == "(coef)":
+            prefix = "* "
+            suffix = ""
+        else:
+            suffix = " " + suffix
+
+        self.fLabelPrefix = prefix
+        self.fLabelSuffix = suffix
+        self.fBar.setSuffixes(prefix, suffix)
 
     def setName(self, name):
         self.fName = name
@@ -441,6 +454,10 @@ class ParamSpinBox(QAbstractSpinBox):
 
         self.fBox.currentIndexChanged['QString'].connect(self.slot_comboBoxIndexChanged)
 
+    def setToolTip(self, text):
+        self.fBar.setToolTip(text)
+        QAbstractSpinBox.setToolTip(self, text)
+
     def stepBy(self, steps):
         if steps == 0 or self.fValue is None:
             return
@@ -557,16 +574,21 @@ class ParamSpinBox(QAbstractSpinBox):
 
         elif actSel == actSet:
             dialog = CustomInputDialog(self, self.fName, self.fValue, self.fMinimum, self.fMaximum,
-                                             self.fStep, self.fStepSmall, self.fScalePoints)
+                                             self.fStep, self.fStepSmall, self.fScalePoints,
+                                             self.fLabelPrefix, self.fLabelSuffix)
             if dialog.exec_():
                 value = dialog.returnValue()
                 self.setValue(value)
 
     @pyqtSlot()
     def slot_updateProgressBarGeometry(self):
-        self.fBar.setGeometry(self.lineEdit().geometry())
+        geometry = self.lineEdit().geometry()
+        dx = geometry.x()-1
+        dy = geometry.y()-1
+        geometry.adjust(-dx, -dy, dx, dy)
+        self.fBar.setGeometry(geometry)
         if self.fUseScalePoints:
-            self.fBox.setGeometry(self.lineEdit().geometry())
+            self.fBox.setGeometry(geometry)
 
     def _getNearestScalePoint(self, realValue):
         finalValue = 0.0
